/******************************************************************************
*
* project name:    Kalaha
* file name:       definitions21.h
* initial date:    17.10.2004
* author:          mhubi@freesurf.ch
* description:     all defintions
* last changes:		 17.10.2004 22:00
*
*******************************************************************************
*
* Kalaha -- an board-game with alpha-beta-pruning-ai
* Copyright (C) 2004  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/


//more general #defines to find under project/options

//use rom call optimisation
#define OPTIMIZE_ROM_CALLS
//avoid a critical bug
#define SET_FILE_IN_USE_BIT

//own object-like macros

//grayscale-planes
//tigcclib: gray.h
#define LPLANE					GrayGetPlane(0)
#define DPLANE					GrayGetPlane(1)

//pot-values
#define POT_HEIGHT1			32
#define POT_HEIGHT2			64
#define POT_WIDTH				26

//key-values
#define K_F2						-10
#define K_ESC						-1
#define K_LEFT					1
#define K_RIGHT					2
#define K_UP						3
#define K_DOWN					4
#define K_ENTER					10
#define K_PLUS					20
#define K_MINUS					21

//status-values
#define ERROR						-20
#define FASTQUIT				-10
#define QUIT						-1
#define NORMAL					0
#define EXECMOVE				10
#define P0_WON					1
#define P1_WON					2
#define DRAWGAME				3
#define CUTOFF					20
#define ENDNODE					21
#define NEXTNODE				22

//player-type
#define P_HUMAN						1
#define P_CALC						2

//fields
#define F_REAL						0
#define F_D1							1

//hiscore-definition
#define MAX_HISCORES			5


//enumeration of the menus
enum Menus {M_EXIT, M_AI, M_1STPLAYER, M_SPEED, M_PLTYPE, M_PL1, M_PL2, M_1STSCREEN, M_PLAY,
						M_OPTSAVED, M_ERRORSAVE, M_OPTLOADED, M_ERRORLOAD, M_GAMESAVED, M_GAMELOADED,
						M_OPTIONS, M_STONES, M_SCORE, M_NOHISCORE, M_HISCORE, M_NAME, M_P0WON, M_P1WON, M_TIED,
						M_HIGHTABLE, M_HELP, M_KEYCONFIG, M_ABOUT, M_RULE};


//own structures

//structure for the data of a player
typedef struct {
  
  unsigned int type;
  unsigned int arr_pos;
  unsigned int ai_strength;
  
} PLAYER;

//structure for the data of the game
typedef struct {
	
	unsigned int cur_pl;
	unsigned int cur_pos;
	int status;
	unsigned int speed;
	unsigned int stones;
	unsigned int moves;
	unsigned int rule;
	
} GAME;

//structure for the data of the ai
typedef struct {
	
	unsigned int player;
	unsigned int ennemi;
	unsigned int next_pl;
	unsigned int end_node;
	unsigned int evaluate;
	int max_depth;
	unsigned int end_move;
	
} AI;

//structure for the data of the menus
typedef struct {
	
	unsigned int max_length;
	unsigned int menu_points;
	unsigned int width;
	unsigned int height;
	unsigned int x;
	unsigned int y;
	unsigned int str_start;
	
} MENU;

//structure to save the options
typedef struct {
	
	PLAYER player[2];
	GAME game;
	
} SAVE_OPTIONS;

//structure to save the game
typedef struct {
	
	PLAYER player[2];
	GAME game;
	unsigned int pot[15];
	
} SAVE_GAME;

//structure of a hiscore-entry
typedef struct {
	char name[12];
	unsigned int score;
} SCORE;


//functions-declarations (-->prototypes)

//main.c
void LeadOut(void);
void _main(void);

//screen.c
void DelScreen(void);
void DrawEntireField(void);
void DrawBackground(void);
void DelCorners(void *, unsigned int , unsigned int , unsigned int , unsigned int);
void DrawPots(void);
void ClearPot(unsigned int);
void DrawArrow(void);
void DelArrow(void);
void DrawStones(unsigned int);
void DrawHiscores(SCORE *);
void MsgDelMiddle(void);
void MsgDrawAiNodes(void);
void MsgDrawAidepth(void);
void MsgDrawAiEnd(int);

//input.c
void Wait(void);
void WaitKey(void);
int KeyPressed(void);
void InputStr(char *, unsigned int);

//game-routines.c
void GameLoop(void);
void InitGame(void);
void InitPots(void);
void ExecMove1(unsigned int);
void ExecMove2(unsigned int);
void CheckRules1(unsigned int);
void CheckRules2(unsigned int);
void CheckWinner(void);
void NextRound(void);

//ai.c
void DynamicAi(void);
int MiniMax(int, unsigned int, int, int);
void AiExecMove1(unsigned int, unsigned int, unsigned int);
void AiExecMove2(unsigned int, unsigned int, unsigned int);
void AiCheckRules1(unsigned int, unsigned int, unsigned int);
void AiCheckRules2(unsigned int, unsigned int, unsigned int);
int AiCheckWinner(unsigned int, int, unsigned int *);

//gui.c
void MoveArrow(int);
void MenuMainRoutine(unsigned int);
void MenuDrawWindow(MENU *);
void MenuDrawText(MENU *);
void MenuDrawArrow(MENU *, unsigned int);
void MenuDelArrow(MENU *, unsigned int);
void MenuMakeChoice(MENU *, unsigned int *);
void Nothing(void);
void Error(void);
void NewGame(void);
void End(void);
void FirstPlayer(void);
void MenuOptions(void);
void MenuSpeed(void);
void MenuStones(void);
void MenuRule(void);
void Help(void);
void HelpPages(void);
void CalcScore(void);
void CalcHiscores(unsigned int);

//extern.c
void Memory(void);
unsigned char *GetDataPtr(const char *, short);
void Archive(const char *);
void Unarchive(const char *);
void SaveOptions(void);
void LoadOptions(void);
void SaveGame(void);
void LoadGame(void);
unsigned int NeedHiscoreFile(void);
unsigned int CreateHiscoreFile(void);
unsigned int SaveHiscores(SCORE *);
unsigned int LoadHiscores(SCORE *);


//=============================================================================
// file history
//=============================================================================
//
//	17.20.2004	pasted all the definition-stuff from the main-h-file into here
